/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2014  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Gio = imports.gi.Gio;

const Config = imports.misc.config;
const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const CurrentExtension = ExtensionUtils.getCurrentExtension();

const Convenience = CurrentExtension.imports.convenience;
const PanelTools = CurrentExtension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - Notifications
 */
const StatusArea = Main.panel._statusArea || Main.panel.statusArea;

const KEY_SHOW_NOTIFICATIONS = 'show-notifications';

let _notificationsIndicator;

let _prototypePushNotification;

function _overridePrototype(enable) {
    const MessageTray = imports.ui.messageTray;

    if (enable) {
        _prototypePushNotification = MessageTray.Source.prototype.pushNotification;
        MessageTray.Source.prototype.pushNotification = function(notification) {
            _prototypePushNotification.call(this, notification);
            _notificationsIndicator._onShowNotifications();
        };
    } else {
        MessageTray.Source.prototype.pushNotification = _prototypePushNotification;
    }
}

const PopupNotificationsMenuItem = new Lang.Class({
    Name: 'PopupNotificationsMenuItem',
    Extends: PanelTools.PopupMenuItem,

    _init: function(app, status) {
        this._app = app;
        this.parent(this._app.title, true, { reactive: false });
        let iconName = null;
        if (!iconName) try { iconName = this._app.mount.get_icon().get_names()[0]; } catch(e) {}
        if (!iconName) try { iconName = this._app.getSummaryIcon().get_children()[0].get_child().get_icon_name(); } catch(e) {}
        if (!iconName) iconName = 'dialog-information';
        if (iconName) this.setIconName(iconName);
        this.status.text = status;
        this.addButton('edit-clear-symbolic', Lang.bind(this, this.destroy));
        this._app.connect('destroy', Lang.bind(this, this.destroy));
    }
});

const NotificationsIndicator = new Lang.Class({
    Name: 'NotificationsIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        _notificationsIndicator = this;
        this.parent(0.0, _("Notifications"), { icon_name: 'dialog-warning' });
        this._notifications = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._notifications);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemClear = new PanelTools.PopupMenuItem(_("Clear Notifications"), true);
        this._itemClear.setIconName('edit-clear');
        this.menu.addMenuItem(this._itemClear);
        this._itemClear.connect('activate', Lang.bind(this, function() {
            if (this.menu.isOpen) this.menu.close(0);
            this.actor.hide();
            this._notifications.removeAll();
        }));
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_NOTIFICATIONS, Lang.bind(this, this._onShowNotifications));
        this._onShowNotifications();
        _overridePrototype(true);
        this.actor.connect('destroy', Lang.bind(this, function() {
            _overridePrototype(false);
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _onShowNotifications: function() {
        let visible = this._settings.get_boolean(KEY_SHOW_NOTIFICATIONS);
        if (visible) {
            this._updateNotifications();
            if (this._notifications.numMenuItems === 0) visible = false;
        }
        this.actor.visible = visible;
    },

    _updateNotifications: function() {
        if (this.menu.isOpen) this.menu.close(0);
        let apps = [];
        if (Main.messageTray.getSources) {
            apps = Main.messageTray.getSources();
        } else if (Main.messageTray._summaryItems) {
            apps = Main.messageTray._summaryItems;
        }
        for (let i = 0; i < apps.length; i++) {
            let app = apps[i];
            if (app.source) app = app.source;
            if (app._mounts) continue;
            if (app.notifications[0]) {
                let status = null;
                if (app.notifications.length > 1) {
                    status = app.notifications.length.toString();
                } else {
                    status = app.notifications[0].title;
                }
                if (status) {
                    let newNotify = true;
                    let menuItems = this._notifications._getMenuItems();
                    for each(let item in menuItems) {
                        if (item.label.text === app.title) {
                            item.status.text = status;
                            newNotify = false;
                        }
                    }
                    if (newNotify) {
                        let item = new PopupNotificationsMenuItem(app, status);
                        item.connect('destroy', Lang.bind(this, function() {
                            if (this._notifications.numMenuItems < 1) {
                                if (this.menu.isOpen) this.menu.close(0);
                                this.actor.hide();
                            }
                        }));
                        this._notifications.addMenuItem(item);
                    }
                }
            }
        }
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfNotifications', new NotificationsIndicator(), PanelTools.getChildren('right') - 1, 'right');
}
