/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2014  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;
const Soup = imports.gi.Soup;

const ExtensionUtils = imports.misc.extensionUtils;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    Weather - Preferences
 */
// Soup session (see https://bugzilla.gnome.org/show_bug.cgi?id=661323#c64) (Simon Legner)
const _httpSession = new Soup.SessionAsync();
Soup.Session.prototype.add_feature.call(_httpSession, new Soup.ProxyResolverDefault());

const KEY_LOCATION = 'location';
const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_SETTINGS = 'show-settings';
const KEY_SHOW_WEATHER_FORECAST = 'show-weather-forecast';
const KEY_WEATHER_FORECAST = 'weather-forecast';
const KEY_WEATHER_ICONS = 'weather-icons';
const KEY_WEATHER_LOCATIONS = 'weather-locations';
const KEY_WEATHER_UNITS = 'weather-units';
const KEY_WIND_DIRECTION = 'wind-direction';

const SETTINGS_GLOBAL = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_REFRESH_INTERVAL,
        label:   _("Refresh interval"),
        tooltip: _("Select the desired refresh interval."),
        values:  { '300':  _("5 Minutes"),
                   '600':  _("10 Minutes"),
                   '900':  _("15 Minutes"),
                   '1800': _("30 Minutes"),
                   '3600': _("1 Hour"),
                   '7200': _("2 Hours") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_WEATHER_UNITS,
        label:   _("System of Units"),
        tooltip: _("Select the system of units for weather."),
        values:  { 'default': _("Anglo American"),
                   'metric':  _("Metric") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'tree',
        key1:    KEY_WEATHER_LOCATIONS,
        key2:    KEY_LOCATION,
        label1:  _("Location"),
        label2:  _("WOEID"),
        buttons: { 'add': _("Add Location"),
                   'del': _("Remove Location") },
        dialog:  WeatherLocationSearch
    }
];

const SETTINGS_DISPLAY = [
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_WEATHER_FORECAST,
        label:   _("Weather Forecast"),
        tooltip: _("Select the number of days for weather forecast."),
        values:  { '1 day':  _("1 Day"),
                   '2 days': _("2 Days"),
                   '3 days': _("3 Days"),
                   '4 days': _("4 Days") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_WEATHER_ICONS,
        label:   _("Weather Icons"),
        tooltip: _("Select the desired icons for Weather."),
        values:  { 'default': _("Default"),
                   'system':  _("System icons") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'combo',
        key:     KEY_WIND_DIRECTION,
        label:   _("Wind Direction"),
        tooltip: _("Select the display mode of wind direction."),
        values:  { 'arrows':  _("Arrows"),
                   'letters': _("Letters") }
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_WEATHER_FORECAST,
        label:   _("Show Weather Forecast"),
        tooltip: _("If true, display Weather Forecast.")
    },
    {   schema:  Convenience.CURRENT_SETTINGS_SCHEMA,
        type:    'switcher',
        key:     KEY_SHOW_SETTINGS,
        label:   _("Show Settings"),
        tooltip: _("If true, display Settings for Weather.")
    }
];

function getSorted(a, b) {
    if (a.toUpperCase() === b.toUpperCase()) return 0;
    if (a.toUpperCase() > b.toUpperCase()) return 1;
    return -1;
}

function WeatherLoadData(url, fun) {
    let message = Soup.Message.new('GET', url);
    _httpSession.queue_message(message, Lang.bind(this, function(_httpSession, message) {
        if (!message.response_body.data) fun.call(this, 0);
        try {
            let jp = JSON.parse(message.response_body.data);
            fun.call(this, jp);
        } catch(e) {
            fun.call(this, false);
        }
    }));
}

function WeatherLocationSearch(schema, key) {
    let dialog = new Gtk.Dialog({ title: _("Location search"), border_width: 5, modal: true });
    let buttonCANCEL = dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL);
    let buttonOK = dialog.add_button(Gtk.STOCK_OK, Gtk.ResponseType.OK);
    buttonOK.sensitive = 0;
    dialog.set_default_response(Gtk.ResponseType.OK);
    let grid = new Gtk.Grid({ border_width: 5, margin_bottom: 5, column_spacing: 5 });
    grid.attach(new Gtk.Label({ label: _("Location:"), xalign: 0, width_request: 130 }), 0, 0, 1, 1);
    let entry = new Gtk.Entry({ activates_default: true, width_request: 300 });
    grid.attach(entry, 1, 0, 1, 1);
    dialog.get_content_area().add(grid);
    let entryCompletion = new Gtk.EntryCompletion();
    entry.set_completion(entryCompletion);
    let completionModel = Gtk.ListStore.new([ GObject.TYPE_STRING ]);
    entryCompletion.set_model(completionModel);
    entryCompletion.set_text_column(0);
    entryCompletion.set_popup_single_match(true);
    entryCompletion.set_minimum_key_length(1);
    entryCompletion.set_match_func(function(entryCompletion, key, iter) {
        if (iter && completionModel.get_value(iter, 0)) return true;
        return false;
    });
    let testLocation = function(location) {
        buttonOK.sensitive = 0;
        if (location.search(/\[/) === -1 || location.search(/\]/) === -1) return false;
        let woeid = location.split(/\[/)[1].split(/\]/)[0];
        if (!woeid) return false;
        WeatherLoadData(encodeURI('https://query.yahooapis.com/v1/public/yql?format=json&q=select woeid,name,admin1,country from geo.places where woeid="'+woeid+'" limit 1'), function() {
            if (typeof arguments[0].query === 'undefined') return;
            if (Number(arguments[0].query.count) < 1) return;
            let city = arguments[0].query.results.place;
            city = city.woeid+'>'+city.name+(city.admin1 ? ', '+city.admin1.content : '')+(city.country ? ' ('+city.country.code+')' : '');
            let strArray = schema.get_strv(key);
            for (let i = 0; i < strArray.length; i++) {
                if (strArray[i] === city) return;
            }
            buttonOK.sensitive = 1;
        });
        return true;
    };
    let searchLocation = function() {
        let location = entry.get_text();
        if (!testLocation(location)) WeatherLoadData(encodeURI('https://query.yahooapis.com/v1/public/yql?format=json&q=select woeid,name,admin1,country from geo.places where text="*'+location+'*" or text="'+location+'" or woeid="'+location+'"'), function() {
            if (arguments[0]) {
                let city = arguments[0].query;
                if (Number(city.count) > 0) {
                    city = city.results.place;
                    completionModel.clear();
                    for (let i in city) {
                        completionModel.set_value(completionModel.append(), 0, city[i].name+(city[i].admin1 ? ', '+city[i].admin1.content : '')+(city[i].country ? ' ('+city[i].country.code+')' : '')+' ['+city[i].woeid+']');
                    }
                    entryCompletion.complete();
                }
            }
        });
    };
    dialog.connect('response', function(dialog, response_id) {
        if (response_id === Gtk.ResponseType.OK) {
            if (entry.get_text().search(/\[/) === -1 || entry.get_text().search(/\]/) === -1) return;
            let woeid = entry.get_text().split(/\[/)[1].split(/\]/)[0];
            if (!woeid) return;
            WeatherLoadData(encodeURI('https://query.yahooapis.com/v1/public/yql?format=json&q=select woeid,name,admin1,country from geo.places where woeid="'+woeid+'" limit 1'), function() {
                if (typeof arguments[0].query === 'undefined') return;
                if (Number(arguments[0].query.count) < 1) return;
                let city = arguments[0].query.results.place;
                let strArray = schema.get_strv(key);
                strArray.push(city.woeid+'>'+city.name+(city.admin1 ? ', '+city.admin1.content : '')+(city.country ? ' ('+city.country.code+')' : ''));
                strArray.sort(function(a, b) {
                    return getSorted(a.split('>')[1], b.split('>')[1]);
                });
                schema.set_strv(key, strArray);
                dialog.destroy();
            });
            return
        }
        dialog.destroy();
    });
    entry.connect('changed', searchLocation);
    dialog.show_all();
}

const Preferences = new GObject.Class({
    Name: 'WeatherPreferences',
    GTypeName: 'WeatherPreferences',
    Extends: Gtk.Box,

    _init: function() {
        this.parent({ orientation: Gtk.Orientation.VERTICAL, border_width: 10, spacing: 10 });
        this._createPrefsWidget();
    },

    _createPrefsWidget: function() {
        let box;
        let notebook = new Gtk.Notebook({ scrollable: true, height_request: 480, width_request: 640 });
        this.add(notebook, false, false, 0);
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, border_width: 10 });
        for (i in SETTINGS_GLOBAL) {
            if (SETTINGS_GLOBAL[i]['type'] === 'tree') {
                box.pack_start(this._createWidget(SETTINGS_GLOBAL[i]), true, true, 0);
            } else {
                box.pack_start(this._createWidget(SETTINGS_GLOBAL[i]), false, false, 5);
            }
        }
        notebook.append_page(box, new Gtk.Label({ label: _("Generally") }));
        box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, border_width: 10 });
        for (i in SETTINGS_DISPLAY) {
            box.pack_start(this._createWidget(SETTINGS_DISPLAY[i]), false, false, 5);
        }
        notebook.append_page(box, new Gtk.Label({ label: _("Display") }));
    },

    _createWidget: function(args) {
        let box;
        let settings = Convenience.getSettings(args['schema']);
        let widget;
        if (args['type'] === 'tree') {
            let strArray = [];
            box = new Gtk.Frame({ expand: true, margin_bottom: 5, margin_top: 10 });
            widget = new Gtk.Grid({ orientation: Gtk.Orientation.VERTICAL });
            let widgetScrolledWindow = new Gtk.ScrolledWindow({ expand: true });
            widget.add(widgetScrolledWindow);
            let widgetListStore = new Gtk.ListStore();
            widgetListStore.set_column_types([ GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_STRING ]);
            let widgetTreeView = new Gtk.TreeView({ expand: true, model: widgetListStore});
            widgetScrolledWindow.add(widgetTreeView);
            widgetTreeView.get_selection().set_mode(Gtk.SelectionMode.SINGLE);
            let widgetColumnWOEID = new Gtk.TreeViewColumn({ expand: true, sort_column_id: 1, title: _(args['label1']) });
            let widgetRenderer = new Gtk.CellRendererText();
            widgetColumnWOEID.pack_start(widgetRenderer, true);
            widgetColumnWOEID.add_attribute(widgetRenderer, 'text', 1);
            widgetTreeView.insert_column(widgetColumnWOEID, 0);
            let widgetColumnLocation = new Gtk.TreeViewColumn({ expand: true, sort_column_id: 2, title: _(args['label2']) });
            widgetColumnLocation.pack_start(widgetRenderer, true);
            widgetColumnLocation.add_attribute(widgetRenderer, 'text', 2);
            widgetTreeView.insert_column(widgetColumnLocation, 1);
            let widgetSelection = widgetTreeView.get_selection();
            let updateListStore = function() {
                strArray = settings.get_strv(args['key1']);
                widgetListStore.clear();
                for (let i = 0; i < strArray.length; i++) {
                    let item = strArray[i].split('>');
                    widgetListStore.set(widgetListStore.append(), [ 0, 1, 2 ], [ strArray[i], item[1], item[0] ]);
                }
                widgetSelection.select_path(Gtk.TreePath.new_from_string(String(settings.get_int(args['key2']))));
            }
            updateListStore();
            widgetSelection.connect('changed', function() {
                strArray = settings.get_strv(args['key1']);
                let [ any, model, iter ] = widgetTreeView.get_selection().get_selected();
                if (any) {
                    let item = widgetListStore.get_value(iter, 0);
                    let index = strArray.map(function(i) {
                        return i;
                    }).indexOf(item);
                    if (index < 0) return;
                    settings.set_int(args['key2'], parseInt(index));
                }
            });
            settings.connect('changed::'+args['key1'], updateListStore);
            if (args['buttons']) {
                let widgetToolbar = new Gtk.Toolbar();
                widgetToolbar.get_style_context().add_class(Gtk.STYLE_CLASS_PANE_SEPARATOR);
                widget.add(widgetToolbar);
                if (args['buttons']['add']) {
                    let addButton = new Gtk.ToolButton({ stock_id: Gtk.STOCK_NEW, label: _(args['buttons']['add']), is_important: true });
                    addButton.connect('clicked', function() {
                        args['dialog'](settings, args['key1']);
                    });
                    widgetToolbar.add(addButton);
                }
                if (args['buttons']['del']) {
                    let delButton = new Gtk.ToolButton({ stock_id: Gtk.STOCK_DELETE, label: _(args['buttons']['del']), is_important: true });
                    delButton.connect('clicked', function() {
                        strArray = settings.get_strv(args['key1']);
                        let [ any, model, iter ] = widgetTreeView.get_selection().get_selected();
                        if (any && strArray.length > 1) {
                            let item = widgetListStore.get_value(iter, 0);
                            let index = strArray.map(function(i) {
                                return i;
                            }).indexOf(item);
                            if (index < 0) return;
                            strArray.splice(index, 1);
                            settings.set_strv(args['key1'], strArray);
                        }
                    });
                    widgetToolbar.add(delButton);
                }
            }
        } else {
            box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
            switch (args['type']) {
                case 'combo':
                    widget = new Gtk.ComboBoxText();
                    for (value in args['values']) {
                        widget.append(value, _(args['values'][value]));
                    }
                    widget.set_active_id(settings.get_string(args['key']));
                    widget.width_request = 200;
                    widget.connect('changed', function() {
                        settings.set_string(args['key'], widget.get_active_id());
                    });
                    settings.connect('changed::'+args['key'], function() {
                        widget.set_active_id(settings.get_string(args['key']));
                    });
                    break;
                case 'switcher':
                    widget = new Gtk.Switch({ active: settings.get_boolean(args['key']) });
                    widget.connect('notify::active', function() {
                        settings.set_boolean(args['key'], widget.get_active());
                    });
                    settings.connect('changed::'+args['key'], function() {
                        widget.set_active(settings.get_boolean(args['key']));
                    });
                    break;
            }
            if (args['tooltip']) widget.set_tooltip_text(_(args['tooltip']));
            box.pack_start(new Gtk.Label({ label: _(args['label']), xalign: 0 }), true, true, 0);
        }
        box.add(widget);
        return box;
    }
});
